/*  
 * Actipro Software's WPF DataGrid / Editors Integration Add-on
 * http://actipro.codeplex.com/
 * 
 * Copyright (c) 2009-2013 Actipro Software LLC. 
 *  
 * This source code is subject to the terms of the Microsoft Public License (Ms-PL). 
 *  
 * Redistribution and use in source and binary forms, with or without modification, 
 * is permitted provided that redistributions of the source code retain the above 
 * copyright notices and this file header. 
 *  
 * Additional copyright notices should be appended to the list above. 
 * 
 * For details, see <http://www.opensource.org/licenses/ms-pl.html>. 
 *  
 * All other rights reserved. 
 *
 */  


using System.Diagnostics.CodeAnalysis;
using System.Windows;
using ActiproSoftware.Windows.Controls.Editors.Primitives;

namespace ActiproSoftware.Windows.Controls.Editors.Interop.DataGrid.Primitives {

	/// <summary>
	/// Represents a base class for data-bound columns for use in a <c>DataGrid</c> that utilizes <see cref="RangeTypeSpecificEditBoxBase{T}"/>-derived controls.
	/// </summary>
	public abstract class DataGridRangeTypeSpecificEditBoxColumnBase<T> : DataGridTypeSpecificEditBoxColumnBase<T> {

		#region Dependency Properties

		/// <summary>
		/// Identifies the <see cref="Maximum"/> dependency property.  This field is read-only.
		/// </summary>
		/// <value>The identifier for the <see cref="Maximum"/> dependency property.</value>
		[SuppressMessage("Microsoft.Design", "CA1000:DoNotDeclareStaticMembersOnGenericTypes")]
		public static readonly DependencyProperty MaximumProperty = RangeTypeSpecificEditBoxBase<T>.MaximumProperty.AddOwner(typeof(DataGridRangeTypeSpecificEditBoxColumnBase<T>),
			new FrameworkPropertyMetadata(default(T), OnNotifyablePropertyValueChanged));

		/// <summary>
		/// Identifies the <see cref="Minimum"/> dependency property.  This field is read-only.
		/// </summary>
		/// <value>The identifier for the <see cref="Minimum"/> dependency property.</value>
		[SuppressMessage("Microsoft.Design", "CA1000:DoNotDeclareStaticMembersOnGenericTypes")]
		public static readonly DependencyProperty MinimumProperty = RangeTypeSpecificEditBoxBase<T>.MinimumProperty.AddOwner(typeof(DataGridRangeTypeSpecificEditBoxColumnBase<T>),
			new FrameworkPropertyMetadata(default(T), OnNotifyablePropertyValueChanged));

		#endregion // Dependency Properties

		/////////////////////////////////////////////////////////////////////////////////////////////////////
		#region PUBLIC PROCEDURES
		/////////////////////////////////////////////////////////////////////////////////////////////////////

		/// <summary>
		/// Applies standard values to the specified target element.
		/// </summary>
		/// <param name="targetElement">The target element.</param>
		protected override void ApplyStandardValues(FrameworkElement targetElement) {
			if (targetElement is RangeTypeSpecificEditBoxBase<T>) {
				this.ApplyValue(MaximumProperty, targetElement, RangeTypeSpecificEditBoxBase<T>.MaximumProperty);
				this.ApplyValue(MinimumProperty, targetElement, RangeTypeSpecificEditBoxBase<T>.MinimumProperty);
			}
			base.ApplyStandardValues(targetElement);
		}

		/// <summary>
		/// Gets or sets the maximum value allowed.
		/// This is a dependency property.
		/// </summary>
		/// <value>
		/// The maximum value allowed, with <see lanword="null"/> indicating no maximum.
		/// The default value is <c>default(T)</c>.
		/// </value>
		public T Maximum {
			get { return (T)this.GetValue(RangeTypeSpecificEditBoxBase<T>.MaximumProperty); }
			set { this.SetValue(RangeTypeSpecificEditBoxBase<T>.MaximumProperty, value); }
		}

		/// <summary>
		/// Gets or sets the minimum value allowed.
		/// This is a dependency property.
		/// </summary>
		/// <value>
		/// The minimum value allowed, with <see lanword="null"/> indicating no minimum.
		/// The default value is <c>default(T)</c>.
		/// </value>
		public T Minimum {
			get { return (T)this.GetValue(DataGridRangeTypeSpecificEditBoxColumnBase<T>.MinimumProperty); }
			set { this.SetValue(DataGridRangeTypeSpecificEditBoxColumnBase<T>.MinimumProperty, value); }
		}

		#endregion // PUBLIC PROCEDURES
	}
}
